/**
 * Ext GWT 3.0.0-beta3 - Ext for GWT
 * Copyright(c) 2007-2011, Sencha, Inc.
 * licensing@sencha.com
 *
 * http://sencha.com/license
 */
package com.sencha.gxt.widget.core.client.grid;

import java.util.HashMap;
import java.util.Map;

/**
 * Defines the configuration information for an aggregation row. The values for
 * each column can be configured and calculated in several ways. The following
 * methods are supported in order of precedence.
 * 
 * <ol>
 * <li>A static HTML string can be specified.</li>
 * <li>A widget can be specified.</li>
 * <li>A <code>SummaryType</code> can be used to "calculate" a value based on
 * the data. When using a summary type, <b>EITHER</b> a
 * <code>NumberFormat</code> or a <code>AggregationRenderer</code> can be used
 * to "format" the value.
 * </ol>
 * 
 * <p />
 * In some cases, the values will not be calculated by the data stored in the
 * store, for example, with paging. In this situation, there are 2 ways to
 * provide values.
 * 
 * <p />
 * The first method is to specify a <code>AggregationRenderer</code> for the
 * relevant columns. This renderer can any data as needed. The second method is
 * to provide a GXT <code>Model</code> that contains the values for the
 * aggregation row. In this case, the value will be retrieved from the model
 * using the property name associated with the column. With this method, you are
 * responsible for ensuring the model is updated any time the aggregation values
 * are updated.
 * 
 * @param <M> the model type
 */
public class AggregationRowConfig<M> {

  private Map<ColumnConfig<M, ?>, String> cellStyle;
  private Map<ColumnConfig<M, ?>, AggregationRenderer<M>> renderers;

  /**
   * Creates a new aggregation row config.s
   */
  public AggregationRowConfig() {
    renderers = new HashMap<ColumnConfig<M, ?>, AggregationRenderer<M>>();
    cellStyle = new HashMap<ColumnConfig<M, ?>, String>();
  }

  /**
   * Returns the cell style for the given column.
   * 
   * @param id the column id
   * @return the CSS style name
   */
  public String getCellStyle(ColumnConfig<M, ?> id) {
    return cellStyle.get(id);
  }

  /**
   * Returns the aggregation renderer for the given column.
   * 
   * @param id the column id
   * @return the aggregation renderer
   */
  public AggregationRenderer<M> getRenderer(ColumnConfig<M, ?> id) {
    return renderers.get(id);
  }

  /**
   * Sets the cell style for the given column.
   * 
   * @param id the column id
   * @param style the CSS style name
   */
  public void setCellStyle(ColumnConfig<M, ?> id, String style) {
    cellStyle.put(id, style);
  }

  /**
   * Sets the aggregation renderer for the given column.
   * 
   * @param id the column id
   * @param renderer the renderer
   */
  public void setRenderer(ColumnConfig<M, ?> id, AggregationRenderer<M> renderer) {
    renderers.put(id, renderer);
  }

}
